﻿

using ProjectByldozer.Exceptions;

namespace ProjectByldozer.CollectionGenericObjects;

/// <summary>
/// Параметризованный набор объектов
/// </summary>
/// <typeparam name="T">Параметр: ограничение - ссылочный тип</typeparam>
public class ListGenericObjects<T>: ICollectionGenericObjects<T>
    where T : class
{
    /// <summary>
	/// Список объектов, которые храним
	/// </summary>
	private readonly List<T?> _collection;

    /// <summary>
    /// Максимально допустимое число объектов в списке
    /// </summary>
    private int _maxCount;
    public CollectionType GetCollectionType => CollectionType.List;

    public int Count => _collection.Count;

    public int MaxCount
    {
        get => _maxCount;
        set
        {
            if (value > 0)
            {
                _maxCount = value;
            }
        }
    }

    /// <summary>
    /// Конструктор
    /// </summary>
    public ListGenericObjects()
    {
        _collection = new();
    }

    public T? Get(int position)
    {
        //TODO выброс ошибки если выход за границу
        if (position >= Count || position < 0) throw new PositionOutOfCollectionException(position);
        return _collection[position];
    }

    public int Insert(T obj)
    {
        // TODO выброс ошибки если переполнение
        if (Count == _maxCount) throw new CollectionOverflowException(Count);
        _collection.Add(obj);
        return Count;
    }

        public int Insert(T obj, int position)
    {
        // TODO выброс ошибки если переполнение
        // TODO выброс ошибки если за границу 
        if (Count == _maxCount) throw new CollectionOverflowException(Count);
        if (position >= Count || position < 0) throw new PositionOutOfCollectionException(position);
        _collection.Insert(position, obj);
        return position;

    }
      
    public T Remove(int position)
    {
        // TODO если выброс за границу
        if (position >= Count || position < 0) throw new PositionOutOfCollectionException(position);
        T obj = _collection[position];
        _collection.RemoveAt(position);
        return obj;


    }

    public IEnumerable<T?> GetItems()
    {
        for (int i = 0; i < Count; ++i)
        {
            yield return _collection[i];
        }
    }
}

