﻿using ProjectByldozer.Entities;

namespace ProjectByldozer.Drawnings;
/// <summary>
/// Класс, отвечающий за прорисовку и перемещение базового объекта-сущности
/// </summary>

public class DrawningTrackedCar
{
    /// <summary>
    /// Класс-сущность
    /// </summary>
    public EntityTrackedCar? EntityTrackedCar { get; protected set; }

    /// <summary>
    /// Ширина окна
    /// </summary>
    private int? _pictureWidth;

    /// <summary>
    /// Высота окна
    /// </summary>
    private int? _pictureHeight;

    /// <summary>
    /// Левая координата прорисовки бульдозера
    /// </summary>
    protected int? _startPosX;

    /// <summary>
    /// Верхняя кооридната прорисовки  бульдозера
    /// </summary>
    protected int? _startPosY;

    /// <summary>
    /// Ширина прорисовки  бульдозера
    /// </summary>
    private readonly int _drawningTrackedCarWidth = 103;

    /// <summary>
    /// Высота прорисовки  бульдозера
    /// </summary>
    private readonly int _drawningTrackedCarHeight = 75;
    /// <summary>
    /// пустой конструктор
    /// </summary>
    /// 



    /// <summary>
	/// Координата X объекта
	/// </summary>
	public int? GetPosX => _startPosX;

    /// <summary>
    /// Координата Y объекта
    /// </summary>
    public int? GetPosY => _startPosY;

    /// <summary>
    /// Ширина объекта
    /// </summary>
    public int GetWidth => _drawningTrackedCarWidth;

    /// <summary>
    /// Высота объекта
    /// </summary>
    public int GetHeight => _drawningTrackedCarHeight;
    public DrawningTrackedCar()
    {
        _pictureWidth = null;
        _pictureHeight = null;
        _startPosX = null;
        _startPosY = null;
    
    }
    /// <summary>
    /// конструктор
    /// </summary>
    /// <param name="speed">Скорость</param>
    /// <param name="weight">Вес</param>
    /// <param name="bodyColor">Основной цвет</param>
    public DrawningTrackedCar(int speed, double weight, Color bodycolor) : this()
    {
        EntityTrackedCar = new EntityTrackedCar(speed, weight, bodycolor);
    }
    /// <summary>
    /// Конструктор для наследования
    /// </summary>
    /// <param name="_drawningShipWidth"></param>
    /// <param name="_drawnShipHeight"></param>
    protected DrawningTrackedCar(int _drawningShipWidth, int _drawnShipHeight) : this()
    {
        this._drawningTrackedCarWidth = _drawningShipWidth;
        this._drawningTrackedCarHeight = _drawnShipHeight;
    }
    /// <summary>
    /// Конструктор для Drawning
    /// </summary>
    /// <param name="speed"></param>
    /// <param name="weight"></param>
    /// <param name="bodyColor"></param>
    public DrawningTrackedCar(EntityTrackedCar entityTrackedCar)
    {
        EntityTrackedCar = entityTrackedCar;
    }
    /// <summary>
    /// Установка границ поля
    /// </summary>
    /// <param name="width">Ширина поля</param>
    /// <param name="height">Высота поля</param>
    /// <returns>true - границы заданы, false - проверка не пройдена, нельзя разместить объект в этих размерах</returns>
    public bool SetPictureSize(int width, int height)
    {
        if (_drawningTrackedCarWidth < width && _drawningTrackedCarHeight < height)
        {
            // TODO проверка, что объект "влезает" в размеры поля
            // если влезает, сохраняем границы и корректируем позицию объекта, если она была уже установлена
            _pictureWidth = width;
            _pictureHeight = height;
            if (_startPosX.HasValue && _startPosY.HasValue)
            {
                if (_startPosX + _drawningTrackedCarWidth <= _pictureWidth)
                {
                    _startPosX = 0;

                }
                if (_startPosY + _drawningTrackedCarHeight <= _pictureHeight)
                {
                    _startPosY = 0;
                }
            }
            return true;
        }
        return false;
    }

    /// <summary>
    /// Установка позиции
    /// </summary>
    /// <param name="x">Координата X</param>
    /// <param name="y">Координата Y</param>
    public void SetPosition(int x, int y)
    {
        if (!_pictureHeight.HasValue || !_pictureWidth.HasValue)
        {
            return;
        }
        // Проверка выхода за границы формы по оси X 
        if (x < 0) // Если новая позиция находится левее левой границы формы 
        {
            x = 0; // Устанавливаем позицию по оси X равной нулю (левой границе формы) 
        }
        else if (x + _drawningTrackedCarWidth > _pictureWidth.Value) // Если новая позиция выходит за правую границу формы 
        {
            x = _pictureWidth.Value - _drawningTrackedCarWidth; // Корректируем позицию по оси X 
        }

        // Проверка выхода за границы формы по оси Y 
        if (y < 0) // Если новая позиция выше верхней границы формы 
        {
            y = 0; // Устанавливаем позицию по оси Y равной нулю (верхней границе формы) 
        }
        else if (y + _drawningTrackedCarHeight > _pictureHeight.Value) // Если новая позиция выходит за нижнюю границу формы 
        {
            y = _pictureHeight.Value - _drawningTrackedCarHeight; // Корректируем позицию по оси Y 
        }


        _startPosX = x;
        _startPosY = y;
    }





    /// <summary>
    /// Изменение направления перемещения
    /// </summary>
    /// <param name="direction">Направление</param>
    /// <returns>true - перемещене выполнено, false - перемещение невозможно</returns>
    public bool MoveTransport(DirectionType direction)
    {
        if (EntityTrackedCar == null || !_startPosX.HasValue || !_startPosY.HasValue)
        {
            return false;
        }

        switch (direction)
        {
            //влево
            case DirectionType.Left:
                if (_startPosX.Value - EntityTrackedCar.Step > 0)
                {
                    _startPosX -= (int)EntityTrackedCar.Step;
                }
                return true;
            //вверх
            case DirectionType.Up:
                if (_startPosY.Value - EntityTrackedCar.Step > 0)
                {
                    _startPosY -= (int)EntityTrackedCar.Step;
                }
                return true;
            // вправо
            case DirectionType.Right:
                if (_startPosX.Value + _drawningTrackedCarWidth + EntityTrackedCar.Step < _pictureWidth)
                {
                    _startPosX += (int)EntityTrackedCar.Step;
                }
                return true;

            //вниз
            case DirectionType.Down:
                if (_startPosY.Value + _drawningTrackedCarHeight + EntityTrackedCar.Step < _pictureHeight)
                {
                    _startPosY += (int)EntityTrackedCar.Step;
                }

                return true;
            default:
                return false;
        }
    }


    ///<summary>
    /// Прорисовка объекта
    /// </summary>
    /// <param name = "g" ></ param >
    public virtual void DrawTransport(Graphics g)
    {
        if (EntityTrackedCar == null || !_startPosX.HasValue || !_startPosY.HasValue)
        {
            return;
        }



        Pen pen = new(Color.Black);
        Brush bodybrush = new SolidBrush(EntityTrackedCar.BodyColor);

        //границы бульдозера и колес
        g.DrawRectangle(pen, _startPosX.Value , _startPosY.Value + 15, 100, 30);
        g.DrawRectangle(pen, _startPosX.Value + 70, _startPosY.Value, 30, 20);

        g.DrawArc(pen, _startPosX.Value  , _startPosY.Value + 40, 30, 40, 130, 100);
        g.DrawArc(pen, _startPosX.Value +73 , _startPosY.Value + 45, 30, 40, 258, 140);

        g.DrawLine(pen, _startPosX.Value + 5, _startPosY.Value + 75, _startPosX.Value + 100, _startPosY.Value + 75);

        g.DrawEllipse(pen, _startPosX.Value + 30, _startPosY.Value + 62, 10, 10);
        g.DrawEllipse(pen, _startPosX.Value + 48, _startPosY.Value + 62, 10, 10);
        g.DrawEllipse(pen, _startPosX.Value + 65, _startPosY.Value + 62, 10, 10);
        g.DrawEllipse(pen, _startPosX.Value + 40, _startPosY.Value + 48, 7, 7);
        g.DrawEllipse(pen, _startPosX.Value + 60, _startPosY.Value + 48, 7, 7);
        g.DrawEllipse(pen, _startPosX.Value + 5, _startPosY.Value + 50, 20, 20);
        g.DrawEllipse(pen, _startPosX.Value + 80, _startPosY.Value + 50, 20, 20);

        //заливка кабины и гусениц
        Brush br = new SolidBrush(Color.Pink);
        g.FillRectangle(br, _startPosX.Value , _startPosY.Value + 15, 100, 30);
        g.FillRectangle(br, _startPosX.Value + 70, _startPosY.Value, 30, 20);
        g.FillEllipse(bodybrush, _startPosX.Value + 5, _startPosY.Value + 50, 20, 20);
        g.FillEllipse(bodybrush, _startPosX.Value + 80, _startPosY.Value + 50, 20, 20);
        g.FillEllipse(bodybrush, _startPosX.Value + 30, _startPosY.Value + 62, 10, 10);
        g.FillEllipse(bodybrush, _startPosX.Value + 48, _startPosY.Value + 62, 10, 10);
        g.FillEllipse(bodybrush, _startPosX.Value + 65, _startPosY.Value + 62, 10, 10);
        g.FillEllipse(bodybrush, _startPosX.Value + 40, _startPosY.Value + 48, 7, 7);
        g.FillEllipse(bodybrush, _startPosX.Value + 60, _startPosY.Value + 48, 7, 7);

    }
}



