﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ProjectByldozer.Entities;

namespace ProjectByldozer.Drawnings;

public static class ExtensionDrawningTrackedCar
{
    /// <summary>
	/// Разделитель для записи информации по объекту в файл
	/// </summary>
	private static readonly string _separatorForObject = ":";

    /// <summary>
    /// Создание объекта из строки
    /// </summary>
    /// <param name="info">Строка с данными для создания объекта</param>
    /// <returns>Объект</returns>
    public static DrawningTrackedCar? CreateDrawningTrackedCar(this string info)
    {
        string[] strs = info.Split(_separatorForObject);
        EntityTrackedCar? Trackedcar = EntityByldozer.CreateEntityByldozer(strs);
        if (Trackedcar != null)
        {
            return new DrawningByldozer(Trackedcar);
        }

        Trackedcar = EntityTrackedCar.CreateEntityTrackedCar(strs);
        if (Trackedcar != null)
        {
            return new DrawningTrackedCar(Trackedcar);
        }

        return null;
    }

    /// <summary>
	/// Получение данных для сохранения в файл
	/// </summary>
	/// <param name="drawningCar">Сохраняемый объект</param>
	/// <returns>Строка с данными по объекту</returns>
	public static string GetDataForSave(this DrawningTrackedCar drawningTrackedCar)
    {
        string[]? array = drawningTrackedCar?.EntityTrackedCar?.GetStringRepresentation();

        if (array == null)
        {
            return string.Empty;
        }

        return string.Join(_separatorForObject, array);
    }
}
