﻿
namespace ProjectByldozer.Entities;

/// <summary>
/// Класс-сущность "Спортивный автомобиль"
/// </summary>
public class EntityByldozer : EntityTrackedCar
{


    /// <summary>
    /// Дополнительный цвет (для опциональных элементов)
    /// </summary>
    public Color AdditionalColor { get; private set; }
    public void setAdditionalColor(Color color)
    {
        AdditionalColor = color;
    }

    /// <summary>
    /// Признак (опция) наличия отвала
    /// </summary>
    public bool Dump { get; private set; }

    /// <summary>
    /// Признак (опция) наличия разрыхлителя
    /// </summary>
    public bool Bakingpowder { get; private set; }

    /// <summary>
    /// Признак (опция) наличия трубы
    /// </summary>
    public bool Pipe { get; private set; }


    /// <summary>
    /// Инициализация полей объекта-класса спортивного автомобиля
    /// </summary>

    /// <param name="speed">Скорость</param>
    /// <param name="weight">Вес автомобиля</param>
    /// <param name="bodyColor">Основной цвет</param>
    /// <param name="additionalColor">Дополнительный цвет</param>
    /// <param name="dump">Признак наличия обвеса</param>
    /// <param name="bakingpowder">Признак наличия антикрыла</param>
    /// <param name="pipe">Признак наличия трубы</param>
    public EntityByldozer(int speed, double weight, Color bodyColor, Color additionalColor, bool dump, bool bakingpowder, bool pipe) : base(speed, weight, bodyColor)
    {
       
        AdditionalColor = additionalColor;
        Dump = dump;
        Bakingpowder = bakingpowder;
        Pipe = pipe;
    }
    /// <summary>
    /// Получение строк со значениями свойств объекта класса-сущности
    /// </summary>
    /// <returns></returns>
    public override string[] GetStringRepresentation()
    {
        return new[] { nameof(EntityByldozer), Speed.ToString(), Weight.ToString(), BodyColor.Name, AdditionalColor.Name,
            Dump.ToString(), Bakingpowder.ToString(), Pipe.ToString() };
    }

    /// <summary>
	/// Создание объекта из массива строк
	/// </summary>
	/// <param name="strs"></param>
	/// <returns></returns>
	public static EntityByldozer? CreateEntityByldozer(string[] strs)
    {
        if (strs.Length != 8 || strs[0] != nameof(EntityByldozer))
        {
            return null;
        }
        return new EntityByldozer(Convert.ToInt32(strs[1]), Convert.ToDouble(strs[2]), Color.FromName(strs[3]), Color.FromName(strs[4]), 
            Convert.ToBoolean(strs[5]), Convert.ToBoolean(strs[6]), Convert.ToBoolean(strs[7]));
    }
}