﻿using Microsoft.Extensions.Logging;
using ProjectByldozer.CollectionGenericObjects;
using ProjectByldozer.Drawnings;
using ProjectByldozer.Exceptions;

namespace ProjectByldozer;
/// <summary>
/// Форма работы с компанией и ее коллекцией
/// </summary>

public partial class FormCarCollection : Form
{
    /// <summary>
	/// Компания
	/// </summary>
	private AbstractCompany? _company = null;
    /// <summary>
    /// Хранилище коллекций
    /// </summary>
    private readonly StorageCollection<DrawningTrackedCar> _storageCollection;
    private readonly ILogger _logger;
    /// <summary>
    /// Конструктор
    /// </summary>
    public FormCarCollection(ILogger<FormCarCollection> logger)
    {
        InitializeComponent();
        _storageCollection = new();
        _logger = logger;
        _logger.LogInformation("Форма загрузилась");
    }

    private void ComboBoxSelectionCompany_SelectedIndexChanged(object sender, EventArgs e)
    {
        panelCompanyTools.Enabled = false;

    }
    /// /// Добавление машины
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonAddTrackedCar_Click(object sender, EventArgs e)
    {
        FormCarConfig form = new();
        // TODO передать метод
        form.Show();
        form.AddEvent(SetTrackedCar);

    }
    /// <summary>
	/// Добавление автомобиля в коллекцию
	/// </summary>
	/// <param name=" Trackedcar"></param>
	private void SetTrackedCar(DrawningTrackedCar? Trackedcar)
    {
        {
            try
            {
                if (_company == null || Trackedcar == null)
                {
                    return;
                }
                if (_company + Trackedcar != -1)
                {
                    MessageBox.Show("Объект добавлен");
                    pictureBox.Image = _company.Show();
                    _logger.LogInformation("Добавлен объект: " + Trackedcar.GetDataForSave());
                }
            }
            catch (ObjectNotFoundException) { }
            catch (CollectionOverflowException ex)
            {
                MessageBox.Show("Не удалось добавить объект");
                _logger.LogError("Ошибка: {Message}", ex.Message);
            }
            catch (PositionOutOfCollectionException ex)
            {
                MessageBox.Show("Выход за границы коллекции");
                _logger.LogError("Ошибка: {Message}", ex.Message);
            }
        }
    }



    private void ButtonDelCar_Click(object sender, EventArgs e)
    {
        if (string.IsNullOrEmpty(maskedTextBox.Text) || _company == null)
        {
            return;
        }

        if (MessageBox.Show("Удалить объект?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
        {
            return;
        }

        int pos = Convert.ToInt32(maskedTextBox.Text);
        try
        {
            if (_company - pos != null)
            {
                MessageBox.Show("Объект удален");
                pictureBox.Image = _company.Show();
                _logger.LogInformation("Удален объект по позиции " + pos);
            }
        }
        catch (Exception ex)
        {
            MessageBox.Show("Не удалось удалить объект");
            _logger.LogError("Ошибка: {Message}", ex.Message);
        }

    }

    private void ButtonGoToCheck_Click(object sender, EventArgs e)
    {
        if (_company == null)
        {
            return;
        }

        DrawningTrackedCar? trackedcar = null;
        int counter = 100;
        try
        {
            while (trackedcar == null)
            {
                trackedcar = _company.GetRandomObject();
                counter--;
                if (counter <= 0)
                {
                    break;
                }
            }
            FormByldozer form = new()
            {
                SetTrackedCar = trackedcar
            };
            form.ShowDialog();
        }
        catch (Exception ex)
        {
            MessageBox.Show(ex.Message, "Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }

    }

    private void ButtonRefresh_Click(object sender, EventArgs e)
    {
        if (_company == null)
        {
            return;
        }

        pictureBox.Image = _company.Show();
    }
    /// <summary>
    /// добавление
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonCjllecyionAdd_Click(object sender, EventArgs e)
    {
        if (string.IsNullOrEmpty(textBoxCollectionName.Text) || (!radioButtonList.Checked && !radioButtonMassiv.Checked))
        {
            MessageBox.Show("Не все данные заполнены", "Ошибка",
            MessageBoxButtons.OK, MessageBoxIcon.Error);
            return;
        }
        try
        {
            CollectionType collectionType = CollectionType.None;
            if (radioButtonMassiv.Checked)
            {
                collectionType = CollectionType.Massive;
            }
            else if (radioButtonList.Checked)
            {
                collectionType = CollectionType.List;
            }
            _storageCollection.AddCollection(textBoxCollectionName.Text, collectionType);
            RerfreshListBoxItems();
            _logger.LogInformation("Коллекция добавлена " + textBoxCollectionName.Text);
        }
        catch (Exception ex)
        {
            _logger.LogError("Ошибка: {Message}", ex.Message);
        }
    }

    private void RerfreshListBoxItems()
    {
        listBoxCollection.Items.Clear();
        for (int i = 0; i < _storageCollection.Keys?.Count; ++i)
        {
            string? colName = _storageCollection.Keys?[i];
            if (!string.IsNullOrEmpty(colName))
            {
                listBoxCollection.Items.Add(colName);
            }
        }
    }
    /// <summary>
    /// удаление коллекции
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonCollectionDel_Click(object sender, EventArgs e)
    {
        if (listBoxCollection.SelectedIndex < 0 || listBoxCollection.SelectedItem == null)
        {
            MessageBox.Show("Коллекция не выбрана");
            return;
        }
        try
        {
            if (MessageBox.Show("Удалить коллекцию?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
            {
                return;
            }
            _storageCollection.DelCollection(listBoxCollection.SelectedItem.ToString());
            RerfreshListBoxItems();
            _logger.LogInformation("Коллекция: " + listBoxCollection.SelectedItem.ToString() + " удалена");
        }
        catch (Exception ex)
        {
            _logger.LogError("Ошибка: {Message}", ex.Message);
        }
    }
    /// <summary>
    /// 
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonCreateCompany_Click(object sender, EventArgs e)
    {
        if (listBoxCollection.SelectedIndex < 0 || listBoxCollection.SelectedItem == null)
        {
            MessageBox.Show("Коллекция не выбрана");
            return;
        }
        ICollectionGenericObjects<DrawningTrackedCar>? collection =
            _storageCollection[listBoxCollection.SelectedItem.ToString() ?? string.Empty];
        if (collection == null)
        {
            MessageBox.Show("Коллекция не проинициализирована");
            return;
        }
        switch (comboBoxSelectionCompany.Text)
        {
            case "Хранилище":
                _company = new TrackedCarGarage(pictureBox.Width, pictureBox.Height, collection);
                break;
        }
        panelCompanyTools.Enabled = true;
        RerfreshListBoxItems();
    }

    private void SaveToolStripMenuItem_Click(object sender, EventArgs e)

    {
        if (saveFileDialog.ShowDialog() == DialogResult.OK)
        {
            try
            {
                _storageCollection.SaveData(saveFileDialog.FileName);
                MessageBox.Show("Сохранение прошло успешно", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
                _logger.LogInformation("Сохранение в файл: {filename}", saveFileDialog.FileName);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);
                _logger.LogError("Ошибка: {Message}", ex.Message);
            }
        }
    }


    private void LoadToolStripMenuItem_Click(object sender, EventArgs e)
    {
        if (openFileDialog.ShowDialog() == DialogResult.OK)
        {
            try
            {
                _storageCollection.LoadData(openFileDialog.FileName);
                MessageBox.Show("Загрузка прошла успешно", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
                RerfreshListBoxItems();
                _logger.LogInformation("Загрузка из файла: {filename}", openFileDialog.FileName);
            }
            catch (Exception ex)
            {
                MessageBox.Show("Не сохранилось", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);
                _logger.LogError("Ошибка: {Message}", ex.Message);
            }
        }
    }

}
