﻿using ProjectByldozer.Drawnings;
using ProjectByldozer.Entities;
namespace ProjectByldozer;
/// <summary>
/// Форма конфигурации объекта
/// </summary>
public partial class FormCarConfig : Form
{
    /// <summary>
    /// Объект - прорисовка автомобиля
    /// </summary>
    private DrawningTrackedCar? _TrackedCar;

    /// <summary>
    /// Событие для передачи объекта
    /// </summary>
    private event  Action<DrawningTrackedCar>? TrackedCarDelegate;
    /// <summary>
    /// конструктор
    /// </summary>
    public FormCarConfig()
    {
        InitializeComponent();
        panelRed.MouseDown += Panel_MouseDown;
        panelGreen.MouseDown += Panel_MouseDown;
        panelBlue.MouseDown += Panel_MouseDown;
        panelYellow.MouseDown += Panel_MouseDown;
        panelWhite.MouseDown += Panel_MouseDown;
        panelGray.MouseDown += Panel_MouseDown;
        panelBlack.MouseDown += Panel_MouseDown;
        panelPurple.MouseDown += Panel_MouseDown;
        // TODO buttonCancel.Click привязать анонимный метод через lambda с закрытием формы
        buttonCancel.Click += (sender, e) => Close();
    }
    /// <summary>
	/// Привязка внешнего метода к событию
	/// </summary>
	/// <param name="TrackedcarDelegate"></param>
	public void AddEvent(Action<DrawningTrackedCar> TrackedcarDelegate)
    {
        TrackedCarDelegate += TrackedcarDelegate;
    }

    private void Panel_MouseDown(object? sender, MouseEventArgs e)
    {
        (sender as Control)?.DoDragDrop((sender as Control).BackColor, DragDropEffects.Move | DragDropEffects.Copy);

        // TODO отправка цвета в Drag&Drop
    }

    


    private void labelBodyColor_DragEnter(object sender, DragEventArgs e)
    {
        if (e.Data.GetDataPresent(typeof(Color)))
        {
            e.Effect = DragDropEffects.Copy;
        }
        else
        {
            e.Effect = DragDropEffects.None;
        }
    }
    private void labelBodyColor_DragDrop(object sender, DragEventArgs e)
    {
        if (_TrackedCar != null)
        {
            _TrackedCar.EntityTrackedCar.setBodyColor((Color)e.Data.GetData(typeof(Color)));
            DrawObject();
        }
    }

    private void labelAdditionalColor_DragEnter(object sender, DragEventArgs e)
    {
        if (_TrackedCar is DrawningByldozer)
        {
            if (e.Data.GetDataPresent(typeof(Color)))
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }
    }

    private void labelAdditionalColor_DragDrop(object sender, DragEventArgs e)
    {
        if (_TrackedCar.EntityTrackedCar is EntityByldozer _byldozer)
        {
            _byldozer.setAdditionalColor((Color)e.Data.GetData(typeof(Color)));
        }
        DrawObject();
    }




    /// <summary>
    /// Прорисовка объекта
    /// </summary>
    private void DrawObject()
    {
        Bitmap bmp = new(pictureBoxObject.Width, pictureBoxObject.Height);
        Graphics gr = Graphics.FromImage(bmp);
        _TrackedCar?.SetPictureSize(pictureBoxObject.Width, pictureBoxObject.Height);
        _TrackedCar?.SetPosition(15, 15);
        _TrackedCar?.DrawTransport(gr);
        pictureBoxObject.Image = bmp;
    }
    /// <summary>
    /// Передаем информацию при нажатии на Label
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void LabelObject_MouseDown(object sender, MouseEventArgs e)
    {

        (sender as Label)?.DoDragDrop((sender as Label)?.Name ?? string.Empty, DragDropEffects.Move | DragDropEffects.Copy);

    }
    /// <summary>
    /// Проверка получаемой информации (ее типа на соответствие требуемому)
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void PanelObject_DragEnter(object sender, DragEventArgs e)
    {
        e.Effect = e.Data?.GetDataPresent(DataFormats.Text) ?? false ? DragDropEffects.Copy : DragDropEffects.None;
    }
    /// <summary>
    /// Действия при приеме перетаскиваемой информации
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void PanelObject_DragDrop(object sender, DragEventArgs e)
    {
        switch (e.Data?.GetData(DataFormats.Text)?.ToString())
        {
            case "labelSimpleObject":
                _TrackedCar = new DrawningTrackedCar((int)numericUpDownSpeed.Value, (double)numericUpDownWeight.Value, Color.White);
                break;
            case "labelModifiedObject":
                _TrackedCar = new DrawningByldozer((int)numericUpDownSpeed.Value, (double)numericUpDownWeight.Value, Color.White,
                    Color.Black, checkBoxDump.Checked, checkBoxBakingpowder.Checked, checkBoxPipe.Checked);
                break;
        }
        labelBodyColor.BackColor = Color.Empty;
        labelAdditionalColor.BackColor = Color.Empty;
        DrawObject();
    }
    /// <summary>
	/// Передача объекта
	/// </summary>
	/// <param name="sender"></param>
	/// <param name="e"></param>
    private void ВuttonAdd_Click(object sender, EventArgs e)
    {
        if (_TrackedCar != null)
        {
            TrackedCarDelegate?.Invoke(_TrackedCar);
            Close();
        }
    }


}

