﻿using ProjectByldozer.Drawnings;

namespace ProjectByldozer.MovementStrategy;

public class MoveableTrackedCar : IMoveableObject
{

    /// <summary>
    /// Поле-объект класса DrawningTrackedCar или его наследника
    /// </summary>
    private readonly DrawningTrackedCar? _Trackedcar = null;

    /// <summary>
    /// Конструктор
    /// </summary>
    /// <param name="Trackedcar">Объект класса DrawningTrackedCar</param>
    public MoveableTrackedCar(DrawningTrackedCar Trackedcar)
    {
        _Trackedcar = Trackedcar;
    }

    public ObjectParameters? GetObjectPosition
    {
        get
        {
            if (_Trackedcar == null || _Trackedcar.EntityTrackedCar == null || !_Trackedcar.GetPosX.HasValue || !_Trackedcar.GetPosY.HasValue)
            {
                return null;
            }
            return new ObjectParameters(_Trackedcar.GetPosX.Value, _Trackedcar.GetPosY.Value, _Trackedcar.GetWidth, _Trackedcar.GetHeight);
        }
    }

    public int GetStep => (int)(_Trackedcar?.EntityTrackedCar?.Step ?? 0);

    public bool TryMoveObject(MovementDirection direction)
    {
        if (_Trackedcar == null || _Trackedcar.EntityTrackedCar == null)
        {
            return false;
        }

        return _Trackedcar.MoveTransport(GetDirectionType(direction));
    }

    /// <summary>
    /// Конвертация из MovementDirection в DirectionType
    /// </summary>
    /// <param name="direction">MovementDirection</param>
    /// <returns>DirectionType</returns>
    private static DirectionType GetDirectionType(MovementDirection direction)
    {
        return direction switch
        {
            MovementDirection.Left => DirectionType.Left,
            MovementDirection.Right => DirectionType.Right,
            MovementDirection.Up => DirectionType.Up,
            MovementDirection.Down => DirectionType.Down,
            _ => DirectionType.Unknow,
        };
    }
}
